@@ -15,6 +15,7 @@
 
 /* Describe, check and repair of MARIA tables */
 
+#define VER "1.3"
 #include "ma_fulltext.h"
 #include <myisamchk.h>
 #include <my_bit.h>
@@ -25,6 +26,7 @@
 /* Remove next line if you want aria_chk to produce a stack trace */
 #undef HAVE_BACKTRACE
 #include <my_stacktrace.h>
+#include <welcome_copyright_notice.h>
 
 static uint decode_bits;
 static char **default_argv;
@@ -79,7 +81,6 @@ static char default_open_errmsg[]=  "%d when opening Aria table '%s'";
 static char default_close_errmsg[]= "%d when closing Aria table '%s'";
 
 static void get_options(int *argc,char * * *argv);
-static void print_version(void);
 static void usage(void);
 static int maria_chk(HA_CHECK *param, char *filename);
 static void descript(HA_CHECK *param, register MARIA_HA *info, char *name);
@@ -145,7 +146,8 @@ int main(int argc, char **argv)
   {
     if ((ma_control_file_open(FALSE, opt_require_control_file ||
                               !(check_param.testflag & T_SILENT),
-                              TRUE)))
+                              TRUE,
+                              control_file_open_flags)))
     {
       if (opt_require_control_file ||
           (opt_transaction_logging && (check_param.testflag & T_REP_ANY)))
@@ -419,25 +421,24 @@ static struct my_option my_long_options[] =
     "Size of page buffer. Used by --safe-repair",
     &check_param.use_buffers, &check_param.use_buffers, 0,
     GET_ULONG, REQUIRED_ARG, PAGE_BUFFER_INIT, 1024L*1024L,
-    SIZE_T_MAX, (long) MALLOC_OVERHEAD, (long) IO_SIZE, 0},
+    SIZE_T_MAX, 0, (long) IO_SIZE, 0},
   { "read_buffer_size", OPT_READ_BUFFER_SIZE,
     "Read buffer size for sequential reads during scanning",
     &check_param.read_buffer_length,
     &check_param.read_buffer_length, 0, GET_ULONG, REQUIRED_ARG,
     (long) READ_BUFFER_INIT, (long) MALLOC_OVERHEAD,
-    ~0ULL, (long) MALLOC_OVERHEAD, (long) 1L, 0},
+    ~0ULL, 0, (long) 1L, 0},
   { "write_buffer_size", OPT_WRITE_BUFFER_SIZE,
     "Write buffer size for sequential writes during repair of fixed size or dynamic size rows",
     &check_param.write_buffer_length,
     &check_param.write_buffer_length, 0, GET_ULONG, REQUIRED_ARG,
     (long) READ_BUFFER_INIT, (long) MALLOC_OVERHEAD,
-    ~0UL, (long) MALLOC_OVERHEAD, (long) 1L, 0},
+    ~0UL, 0, (long) 1L, 0},
   { "sort_buffer_size", OPT_SORT_BUFFER_SIZE,
     "Size of sort buffer. Used by --recover",
     &check_param.orig_sort_buffer_length,
     &check_param.orig_sort_buffer_length, 0, GET_ULL, REQUIRED_ARG,
-    SORT_BUFFER_INIT, MARIA_MIN_SORT_MEMORY, SIZE_T_MAX/10, MALLOC_OVERHEAD,
-    1L, 0},
+    SORT_BUFFER_INIT, MARIA_MIN_SORT_MEMORY, SIZE_T_MAX/10, 0, 1L, 0},
   { "sort_key_blocks", OPT_SORT_KEY_BLOCKS,
     "Internal buffer for sorting keys; Don't touch :)",
     &check_param.sort_key_blocks,
@@ -472,13 +473,6 @@ static struct my_option my_long_options[] =
 };
 
 
-static void print_version(void)
-{
-  printf("%s  Ver 1.3 for %s on %s\n", my_progname, SYSTEM_TYPE,
-	 MACHINE_TYPE);
-}
-
-
 static void usage(void)
 {
   print_version();
@@ -523,7 +517,7 @@ static void usage(void)
 
   puts("Check options (check is the default action for aria_chk):\n\
   -c, --check	      Check table for errors.\n\
-  -e, --extend-check  Check the table VERY throughly.  Only use this in\n\
+  -e, --extend-check  Check the table VERY thoroughly. Only use this in\n\
                       extreme cases as aria_chk should normally be able to\n\
                       find out if the table is ok even without this switch.\n\
   -F, --fast	      Check only tables that haven't been closed properly.\n\
@@ -639,9 +633,7 @@ Recover (repair)/ options (When using '--recover' or '--safe-recover'):\n\
 
 const char *maria_stats_method_names[] = {"nulls_unequal", "nulls_equal",
                                            "nulls_ignored", NullS};
-TYPELIB maria_stats_method_typelib= {
-  array_elements(maria_stats_method_names) - 1, "",
-  maria_stats_method_names, NULL};
+TYPELIB maria_stats_method_typelib= CREATE_TYPELIB_FOR(maria_stats_method_names);
 
 	 /* Read options */
 
@@ -1232,14 +1224,16 @@ static int maria_chk(HA_CHECK *param, char *filename)
   */
   maria_lock_database(info, F_EXTRA_LCK);
   datafile= info->dfile.file;
-  if (init_pagecache(maria_pagecache, (size_t) param->use_buffers, 0, 0,
-                     maria_block_size, 0, MY_WME) == 0)
+  if (multi_init_pagecache(&maria_pagecaches, 1, (size_t) param->use_buffers,
+                           0, 0, maria_block_size, 0, MY_WME))
   {
     _ma_check_print_error(param, "Can't initialize page cache with %lu memory",
                           (ulong) param->use_buffers);
     error= 1;
     goto end2;
   }
+  /* The pagecache is initialized. Update the table pagecaches pointers */
+  ma_change_pagecache(info);
 
   if (param->testflag & (T_REP_ANY | T_SORT_RECORDS | T_SORT_INDEX |
                          T_ZEROFILL))
@@ -1345,7 +1339,8 @@ static int maria_chk(HA_CHECK *param, char *filename)
         */
         my_bool update_index=1;
         for (key=0 ; key < share->base.keys; key++)
-          if (share->keyinfo[key].flag & (HA_BINARY_PACK_KEY|HA_FULLTEXT))
+          if (share->keyinfo[key].flag & HA_BINARY_PACK_KEY ||
+              share->keyinfo[key].key_alg == HA_KEY_ALG_FULLTEXT)
             update_index=0;
 
         error=maria_sort_records(param,info,filename,param->opt_sort_key,
@@ -1370,6 +1365,7 @@ static int maria_chk(HA_CHECK *param, char *filename)
       DBUG_PRINT("info", ("Resetting crashed state"));
       share->state.changed&= ~(STATE_CHANGED | STATE_CRASHED_FLAGS |
                                STATE_IN_REPAIR);
+      share->crash_error= 0;
     }
     else
       maria_mark_crashed(info);
@@ -1434,6 +1430,7 @@ static int maria_chk(HA_CHECK *param, char *filename)
       DBUG_PRINT("info", ("Resetting crashed state"));
       share->state.changed&= ~(STATE_CHANGED | STATE_CRASHED_FLAGS |
                                STATE_IN_REPAIR);
+      share->crash_error= 0;
     }
     else if (!maria_is_crashed(info) &&
              (param->testflag & T_UPDATE_STATE))
@@ -1477,7 +1474,7 @@ end2:
     _ma_check_print_error(param, default_close_errmsg, my_errno, filename);
     DBUG_RETURN(1);
   }
-  end_pagecache(maria_pagecache, 1);
+  multi_end_pagecache(&maria_pagecaches);
   if (error == 0)
   {
     if (param->out_flag & O_NEW_DATA)
@@ -1593,7 +1590,7 @@ static void descript(HA_CHECK *param, register MARIA_HA *info, char *name)
     }
     compile_time_assert((MY_UUID_STRING_LENGTH + 1) <= sizeof(buff));
     buff[MY_UUID_STRING_LENGTH]= 0;
-    my_uuid2str(share->base.uuid, buff);
+    my_uuid2str(share->base.uuid, buff, 1);
     printf("UUID:                %s\n", buff);
     if (ma_control_file_inited() &&
         memcmp(share->base.uuid, maria_uuid, MY_UUID_SIZE))
@@ -1625,6 +1622,8 @@ static void descript(HA_CHECK *param, register MARIA_HA *info, char *name)
 	pos=strmov(pos,"sorted index pages,");
       if (!(share->state.changed & STATE_NOT_ZEROFILLED))
 	pos=strmov(pos,"zerofilled,");
+      if (test_all_bits(share->state.changed, (STATE_NOT_ZEROFILLED | STATE_HAS_LSN)))
+        pos=strmov(pos,"has_lsn,");
       if (!(share->state.changed & STATE_NOT_MOVABLE))
 	pos=strmov(pos,"movable,");
       if (have_control_file && (share->state.changed & STATE_MOVED))
@@ -1697,7 +1696,7 @@ static void descript(HA_CHECK *param, register MARIA_HA *info, char *name)
   {
     keyseg=keyinfo->seg;
     if (keyinfo->flag & HA_NOSAME) text="unique ";
-    else if (keyinfo->flag & HA_FULLTEXT) text="fulltext ";
+    else if (keyinfo->key_alg == HA_KEY_ALG_FULLTEXT) text="fulltext ";
     else text="multip.";
 
     pos=buff;
@@ -1883,7 +1882,7 @@ static int maria_sort_records(HA_CHECK *param,
     param->error_printed=0;
     DBUG_RETURN(0);				/* Nothing to do */
   }
-  if (keyinfo->flag & HA_FULLTEXT)
+  if (keyinfo->key_alg == HA_KEY_ALG_FULLTEXT)
   {
     _ma_check_print_warning(param,"Can't sort table '%s' on FULLTEXT key %d",
                             name,sort_key+1);
